@@ -31,7 +31,6 @@
 # See `gem help install` and `gem help gem_dependencies` for further details.
 
 class Gem::RequestSet::GemDependencyAPI
-
   ENGINE_MAP = { # :nodoc:
     :jruby        => %w[jruby],
     :jruby_18     => %w[jruby],
@@ -51,10 +50,10 @@ class Gem::RequestSet::GemDependencyAPI
     :ruby_21      => %w[ruby rbx maglev truffleruby],
   }.freeze
 
-  mswin     = Gem::Platform.new 'x86-mswin32'
-  mswin64   = Gem::Platform.new 'x64-mswin64'
-  x86_mingw = Gem::Platform.new 'x86-mingw32'
-  x64_mingw = Gem::Platform.new 'x64-mingw32'
+  mswin     = Gem::Platform.new "x86-mswin32"
+  mswin64   = Gem::Platform.new "x64-mswin64"
+  x86_mingw = Gem::Platform.new "x86-mingw32"
+  x64_mingw = Gem::Platform.new "x64-mingw32"
 
   PLATFORM_MAP = { # :nodoc:
     :jruby        => Gem::Platform::RUBY,
@@ -89,14 +88,14 @@ class Gem::RequestSet::GemDependencyAPI
     :truffleruby  => Gem::Platform::RUBY,
     :x64_mingw    => x64_mingw,
     :x64_mingw_20 => x64_mingw,
-    :x64_mingw_21 => x64_mingw
+    :x64_mingw_21 => x64_mingw,
   }.freeze
 
-  gt_eq_0        = Gem::Requirement.new '>= 0'
-  tilde_gt_1_8_0 = Gem::Requirement.new '~> 1.8.0'
-  tilde_gt_1_9_0 = Gem::Requirement.new '~> 1.9.0'
-  tilde_gt_2_0_0 = Gem::Requirement.new '~> 2.0.0'
-  tilde_gt_2_1_0 = Gem::Requirement.new '~> 2.1.0'
+  gt_eq_0        = Gem::Requirement.new ">= 0"
+  tilde_gt_1_8_0 = Gem::Requirement.new "~> 1.8.0"
+  tilde_gt_1_9_0 = Gem::Requirement.new "~> 1.9.0"
+  tilde_gt_2_0_0 = Gem::Requirement.new "~> 2.0.0"
+  tilde_gt_2_1_0 = Gem::Requirement.new "~> 2.1.0"
 
   VERSION_MAP = { # :nodoc:
     :jruby        => gt_eq_0,
@@ -206,7 +205,7 @@ class Gem::RequestSet::GemDependencyAPI
     @git_set            = @set.git_set
     @git_sources        = {}
     @installing         = false
-    @requires           = Hash.new { |h, name| h[name] = [] }
+    @requires           = Hash.new {|h, name| h[name] = [] }
     @vendor_set         = @set.vendor_set
     @source_set         = @set.source_set
     @gem_sources        = {}
@@ -235,7 +234,7 @@ class Gem::RequestSet::GemDependencyAPI
     return unless (groups & @without_groups).empty?
 
     dependencies.each do |dep|
-      @set.gem dep.name, *dep.requirement
+      @set.gem dep.name, *dep.requirement.as_list
     end
   end
 
@@ -380,7 +379,7 @@ class Gem::RequestSet::GemDependencyAPI
         Gem::Requirement.create requirements
       end
 
-    return unless gem_platforms options
+    return unless gem_platforms name, options
 
     groups = gem_group name, options
 
@@ -436,7 +435,7 @@ Gem dependencies file #{@path} requires #{name} more than once.
     reference ||= ref
     reference ||= branch
     reference ||= tag
-    reference ||= 'master'
+    reference ||= "master"
 
     if ref && branch
       warn <<-WARNING
@@ -533,7 +532,7 @@ Gem dependencies file #{@path} includes git reference for both ref/branch and ta
   # Handles the platforms: option from +options+.  Returns true if the
   # platform matches the current platform.
 
-  def gem_platforms(options) # :nodoc:
+  def gem_platforms(name, options) # :nodoc:
     platform_names = Array(options.delete :platform)
     platform_names.concat Array(options.delete :platforms)
     platform_names.concat @current_platforms if @current_platforms
@@ -544,7 +543,7 @@ Gem dependencies file #{@path} includes git reference for both ref/branch and ta
       raise ArgumentError, "unknown platform #{platform_name.inspect}" unless
         platform = PLATFORM_MAP[platform_name]
 
-      next false unless Gem::Platform.match platform
+      next false unless Gem::Platform.match_gem? platform, name
 
       if engines = ENGINE_MAP[platform_name]
         next false unless engines.include? Gem.ruby_engine
@@ -638,8 +637,8 @@ Gem dependencies file #{@path} includes git reference for both ref/branch and ta
   #   :development.  Only one group may be specified.
 
   def gemspec(options = {})
-    name              = options.delete(:name) || '{,*}'
-    path              = options.delete(:path) || '.'
+    name              = options.delete(:name) || "{,*}"
+    path              = options.delete(:path) || "."
     development_group = options.delete(:development_group) || :development
 
     spec = find_gemspec name, path
@@ -698,11 +697,11 @@ Gem dependencies file #{@path} includes git reference for both ref/branch and ta
   def pin_gem_source(name, type = :default, source = nil)
     source_description =
       case type
-      when :default then '(default)'
+      when :default then "(default)"
       when :path    then "path: #{source}"
       when :git     then "git: #{source}"
       when :source  then "source: #{source}"
-      else               '(unknown)'
+      else               "(unknown)"
       end
 
     raise ArgumentError,
@@ -789,7 +788,7 @@ Gem dependencies file #{@path} includes git reference for both ref/branch and ta
     engine_version = options[:engine_version]
 
     raise ArgumentError,
-          'You must specify engine_version along with the Ruby engine' if
+          "You must specify engine_version along with the Ruby engine" if
             engine and not engine_version
 
     return true if @installing
@@ -842,5 +841,4 @@ Gem dependencies file #{@path} includes git reference for both ref/branch and ta
 
     Gem.sources << url
   end
-
 end
